@@ -1,6 +1,21 @@
+##############################################################################
+#
+# Copyright (c) 2007 Zope Foundation and Contributors.
+# All Rights Reserved.
+#
+# This software is subject to the provisions of the Zope Public License,
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
+# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
+# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
+# FOR A PARTICULAR PURPOSE.
+#
+##############################################################################
 import os
 import sys
 
+import six
+
 from zope import i18n
 from zope.security.proxy import ProxyFactory
 
@@ -14,23 +29,35 @@ from z3c.pt import expressions
 
 try:
     from Missing import MV
-    MV  # pyflakes
+
+    MV = MV  # pyflakes pragma: no cover
 except ImportError:
     MV = object()
 
 _marker = object()
 
 
-BOOLEAN_HTML_ATTRS = frozenset([
-    # List of Boolean attributes in HTML that should be rendered in
-    # minimized form (e.g. <img ismap> rather than <img ismap="">)
-    # From http://www.w3.org/TR/xhtml1/#guidelines (C.10)
-    # TODO: The problem with this is that this is not valid XML and
-    # can't be parsed back!
-    "compact", "nowrap", "ismap", "declare", "noshade", "checked",
-    "disabled", "readonly", "multiple", "selected", "noresize",
-    "defer"
-])
+BOOLEAN_HTML_ATTRS = frozenset(
+    [
+        # List of Boolean attributes in HTML that should be rendered in
+        # minimized form (e.g. <img ismap> rather than <img ismap="">)
+        # From http://www.w3.org/TR/xhtml1/#guidelines (C.10)
+        # TODO: The problem with this is that this is not valid XML and
+        # can't be parsed back!
+        "compact",
+        "nowrap",
+        "ismap",
+        "declare",
+        "noshade",
+        "checked",
+        "disabled",
+        "readonly",
+        "multiple",
+        "selected",
+        "noresize",
+        "defer",
+    ]
+)
 
 
 class OpaqueDict(dict):
@@ -39,43 +66,32 @@ class OpaqueDict(dict):
         inst.dictionary = dictionary
         return inst
 
-    @property
-    def __getitem__(self):
-        return self.dictionary.__getitem__
+    def __getitem__(self, name):
+        return self.dictionary[name]
 
-    @property
     def __len__(self):
-        return self.dictionary.__len__
+        return len(self.dictionary)
 
     def __repr__(self):
         return "{...} (%d entries)" % len(self)
 
+
 sys_modules = ProxyFactory(OpaqueDict(sys.modules))
 
 
-class DummyRegistry(object):
-    """This class is for B/W with Chameleon 1.x API."""
-
-    @staticmethod
-    def purge():
-        pass
-
-
 class BaseTemplate(template.PageTemplate):
     content_type = None
     version = 2
 
-    registry = DummyRegistry()
-
     expression_types = {
-        'python': expressions.PythonExpr,
-        'string': StringExpr,
-        'not': NotExpr,
-        'exists': expressions.ExistsExpr,
-        'path': expressions.PathExpr,
-        'provider': expressions.ProviderExpr,
-        'nocall': expressions.NocallExpr,
-        }
+        "python": expressions.PythonExpr,
+        "string": StringExpr,
+        "not": NotExpr,
+        "exists": expressions.ExistsExpr,
+        "path": expressions.PathExpr,
+        "provider": expressions.ProviderExpr,
+        "nocall": expressions.NocallExpr,
+    }
 
     default_expression = "path"
 
@@ -85,22 +101,22 @@ class BaseTemplate(template.PageTemplate):
 
     trim_attribute_space = True
 
+    # https://github.com/zopefoundation/Zope/issues/716
+    enable_comment_interpolation = False
+
     @property
     def boolean_attributes(self):
-        if self.content_type == 'text/xml':
+        if self.content_type == "text/xml":
             return set()
 
         return BOOLEAN_HTML_ATTRS
 
     @property
     def builtins(self):
-        builtins = {
-            'nothing': None,
-            'modules': sys_modules,
-            }
+        builtins = {"nothing": None, "modules": sys_modules}
 
         tales = ExpressionEvaluator(self.engine, builtins)
-        builtins['tales'] = tales
+        builtins["tales"] = tales
 
         return builtins
 
@@ -114,36 +130,47 @@ class BaseTemplate(template.PageTemplate):
     def render(self, target_language=None, **context):
         # We always include a ``request`` variable; it is (currently)
         # depended on in various expression types and must be defined
-        request = context.setdefault('request', None)
+        request = context.setdefault("request", None)
 
         if target_language is None:
             try:
                 target_language = i18n.negotiate(request)
-            except:
+            except Exception:
                 target_language = None
 
-        context['target_language'] = target_language
+        context["target_language"] = target_language
 
         # bind translation-method to request
         def translate(
-            msgid, domain=None, mapping=None,
-            target_language=None, default=None,
-            context=None):
+            msgid,
+            domain=None,
+            mapping=None,
+            target_language=None,
+            default=None,
+            context=None,
+        ):
             if msgid is MV:
                 # Special case handling of Zope2's Missing.MV
                 # (Missing.Value) used by the ZCatalog but is
-                # unhashable
+                # unhashable.
+
+                # This case cannot arise in ordinary templates; msgid
+                # comes from i18n:translate attributes, which does not
+                # take a TALES expression, just a literal string.
+                # However, the 'context' argument is available as an
+                # implementation detail for macros
                 return
             return fast_translate(
-                msgid, domain, mapping, request, target_language, default)
+                msgid, domain, mapping, request, target_language, default
+            )
+
         context["translate"] = translate
 
-        if request is not None and not isinstance(request, basestring):
-            content_type = self.content_type or 'text/html'
+        if request is not None and not isinstance(request, six.string_types):
+            content_type = self.content_type or "text/html"
             response = request.response
             if response and not response.getHeader("Content-Type"):
-                response.setHeader(
-                    "Content-Type", content_type)
+                response.setHeader("Content-Type", content_type)
 
         base_renderer = super(BaseTemplate, self).render
         return base_renderer(**context)
@@ -159,7 +186,7 @@ class BaseTemplate(template.PageTemplate):
             options=kwargs,
             request=request,
             template=self,
-            )
+        )
 
 
 class BaseTemplateFile(BaseTemplate, template.PageTemplateFile):
@@ -175,18 +202,20 @@ class BaseTemplateFile(BaseTemplate, template.PageTemplateFile):
         if not os.path.isabs(filename):
             for depth in (1, 2):
                 frame = sys._getframe(depth)
-                package_name = frame.f_globals.get('__name__', None)
-                if package_name is not None and \
-                       package_name != self.__module__:
+                package_name = frame.f_globals.get("__name__", None)
+                if (
+                    package_name is not None
+                    and package_name != self.__module__
+                ):
                     module = sys.modules[package_name]
                     try:
                         path = module.__path__[0]
                     except AttributeError:
                         path = module.__file__
-                        path = path[:path.rfind(os.sep)]
+                        path = path[: path.rfind(os.sep)]
                     break
                 else:
-                    package_path = frame.f_globals.get('__file__', None)
+                    package_path = frame.f_globals.get("__file__", None)
                     if package_path is not None:
                         path = os.path.dirname(package_path)
                         break
@@ -194,8 +223,7 @@ class BaseTemplateFile(BaseTemplate, template.PageTemplateFile):
             if path is not None:
                 filename = os.path.join(path, filename)
 
-        template.PageTemplateFile.__init__(
-            self, filename, **kwargs)
+        template.PageTemplateFile.__init__(self, filename, **kwargs)
 
         # Set content-type last, so that we can override whatever was
         # magically sniffed from the source template.
@@ -238,21 +266,21 @@ class ViewPageTemplate(PageTemplate):
     is 'path' (standard Zope traversal)."""
 
     def _pt_get_context(self, view, request, kwargs):
-        context = kwargs.get('context')
+        context = kwargs.get("context")
         if context is None:
             context = view.context
-        request = request or kwargs.get('request') or view.request
+        request = request or kwargs.get("request") or view.request
         return dict(
             view=view,
             context=context,
             request=request,
             options=kwargs,
             template=self,
-            )
+        )
 
     def __call__(self, _ob=None, context=None, request=None, **kwargs):
-        kwargs.setdefault('context', context)
-        kwargs.setdefault('request', request)
+        kwargs.setdefault("context", context)
+        kwargs.setdefault("request", request)
         bound_pt = self.bind(_ob)
         return bound_pt(**kwargs)
 
@@ -269,21 +297,28 @@ class BoundPageTemplate(object):
     the class instance on access, which is implemented using this
     helper class."""
 
-    def __init__(self, pt, render):
-        object.__setattr__(self, 'im_self', pt)
-        object.__setattr__(self, 'im_func', render)
+    __self__ = None
+    __func__ = None
 
-    macros = property(lambda self: self.im_self.macros)
-    filename = property(lambda self: self.im_self.filename)
+    def __init__(self, pt, render):
+        object.__setattr__(self, "__self__", pt)
+        object.__setattr__(self, "__func__", render)
+
+    im_self = property(lambda self: self.__self__)
+    im_func = property(lambda self: self.__func__)
+    macros = property(lambda self: self.__self__.macros)
+    filename = property(lambda self: self.__self__.filename)
 
     def __call__(self, *args, **kw):
-        kw.setdefault('args', args)
-        return self.im_func(**kw)
+        kw.setdefault("args", args)
+        return self.__func__(**kw)
 
     def __setattr__(self, name, v):
         raise AttributeError("Can't set attribute", name)
 
     def __repr__(self):
         return "<%s.Bound%s %r>" % (
-            type(self.im_self).__module__,
-            type(self.im_self).__name__, self.filename)
+            type(self.__self__).__module__,
+            type(self.__self__).__name__,
+            self.filename,
+        )
